package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1EncodableVector;
import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1Object;
import cn.com.infosec.asn1.ASN1OctetString;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.ASN1Set;
import cn.com.infosec.asn1.BERConstructedOctetString;
import cn.com.infosec.asn1.BEROutputStream;
import cn.com.infosec.asn1.DERBMPString;
import cn.com.infosec.asn1.DEREncodable;
import cn.com.infosec.asn1.DERNull;
import cn.com.infosec.asn1.DERObject;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.DEROctetString;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.asn1.DERSet;
import cn.com.infosec.asn1.pkcs.AuthenticatedSafe;
import cn.com.infosec.asn1.pkcs.CertBag;
import cn.com.infosec.asn1.pkcs.ContentInfo;
import cn.com.infosec.asn1.pkcs.EncryptedData;
import cn.com.infosec.asn1.pkcs.EncryptedPrivateKeyInfo;
import cn.com.infosec.asn1.pkcs.MacData;
import cn.com.infosec.asn1.pkcs.PKCS12PBEParams;
import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.asn1.pkcs.Pfx;
import cn.com.infosec.asn1.pkcs.PrivateKeyInfo;
import cn.com.infosec.asn1.pkcs.SafeBag;
import cn.com.infosec.asn1.util.ASN1Dump;
import cn.com.infosec.asn1.x509.AlgorithmIdentifier;
import cn.com.infosec.asn1.x509.AuthorityKeyIdentifier;
import cn.com.infosec.asn1.x509.DigestInfo;
import cn.com.infosec.asn1.x509.SubjectKeyIdentifier;
import cn.com.infosec.asn1.x509.X509Extensions;
import cn.com.infosec.asn1.x509.X509ObjectIdentifiers;
import cn.com.infosec.jce.interfaces.BCKeyStore;
import cn.com.infosec.jce.interfaces.PKCS12BagAttributeCarrier;
import cn.com.infosec.util.Arrays;
import cn.com.infosec.util.Strings;
import cn.com.infosec.util.encoders.Hex;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.security.Key;
import java.security.KeyStoreException;
import java.security.KeyStoreSpi;
import java.security.Principal;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.cert.Certificate;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.crypto.Cipher;
import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;

public class JDKPKCS12KeyStore extends KeyStoreSpi implements PKCSObjectIdentifiers, X509ObjectIdentifiers, BCKeyStore {
   private static final int SALT_SIZE = 20;
   private static final int MIN_ITERATIONS = 1024;
   private static final DERObjectIdentifier KEY_ALGORITHM;
   private static final DERObjectIdentifier CERT_ALGORITHM;
   static final int NULL = 0;
   static final int CERTIFICATE = 1;
   static final int KEY = 2;
   static final int SECRET = 3;
   static final int SEALED = 4;
   static final int KEY_PRIVATE = 0;
   static final int KEY_PUBLIC = 1;
   static final int KEY_SECRET = 2;
   private IgnoresCaseHashtable keys;
   private Hashtable localIds;
   private IgnoresCaseHashtable certs;
   private Hashtable chainCerts;
   private Hashtable keyCerts;
   protected SecureRandom random;
   private CertificateFactory certFact;

   static {
      KEY_ALGORITHM = PKCSObjectIdentifiers.pbeWithSHAAnd3_KeyTripleDES_CBC;
      CERT_ALGORITHM = PKCSObjectIdentifiers.pbewithSHAAnd40BitRC2_CBC;
   }

   public JDKPKCS12KeyStore(String provider) {
      Exception var10000;
      label20: {
         super();
         this.keys = new IgnoresCaseHashtable((IgnoresCaseHashtable)null);
         this.localIds = new Hashtable();
         this.certs = new IgnoresCaseHashtable((IgnoresCaseHashtable)null);
         this.chainCerts = new Hashtable();
         this.keyCerts = new Hashtable();
         this.random = new SecureRandom();
         this.certFact = null;
         boolean var10001;
         if (provider != null) {
            try {
               this.certFact = CertificateFactory.getInstance("X.509", provider);
            } catch (Exception var4) {
               var10000 = var4;
               var10001 = false;
               break label20;
            }
         } else {
            try {
               this.certFact = CertificateFactory.getInstance("X.509");
            } catch (Exception var3) {
               var10000 = var3;
               var10001 = false;
               break label20;
            }
         }

         try {
            return;
         } catch (Exception var2) {
            var10000 = var2;
            var10001 = false;
         }
      }

      Exception var5 = var10000;
      throw new IllegalArgumentException("can't create cert factory - " + var5.toString());
   }

   private SubjectKeyIdentifier createSubjectKeyId(PublicKey param1) {
      // $FF: Couldn't be decompiled
   }

   private static byte[] calculatePbeMac(DERObjectIdentifier oid, byte[] salt, int itCount, char[] password, boolean wrongPkcs12Zero, byte[] data) {
      DERObjectIdentifier var10000 = oid;
      SecretKeyFactory var10001 = SecretKeyFactory.getInstance(oid.getId(), "INFOSEC");
      PBEParameterSpec oid1;
      oid1 = new PBEParameterSpec.<init>(salt, itCount);
      JCEPBEKey salt1;
      (salt1 = (JCEPBEKey)var10001.generateSecret(new PBEKeySpec(password))).setTryWrongPKCS12Zero(wrongPkcs12Zero);
      Mac var7 = Mac.getInstance(var10000.getId(), "INFOSEC");
      var7.init(salt1, oid1);
      var7.update(data);
      return var7.doFinal();
   }

   public void setRandom(SecureRandom rand) {
      this.random = rand;
   }

   public Enumeration engineAliases() {
      Hashtable var1;
      var1 = new Hashtable.<init>();
      Enumeration var2 = this.certs.keys();

      while(var2.hasMoreElements()) {
         var1.put(var2.nextElement(), "cert");
      }

      Enumeration var3 = this.keys.keys();

      while(var3.hasMoreElements()) {
         String var4;
         if (var1.get(var4 = (String)var3.nextElement()) == null) {
            var1.put(var4, "key");
         }
      }

      return var1.keys();
   }

   public boolean engineContainsAlias(String alias) {
      return this.certs.get(alias) != null || this.keys.get(alias) != null;
   }

   public void engineDeleteEntry(String alias) {
      Key var2 = (Key)this.keys.remove(alias);
      Certificate var3;
      if ((var3 = (Certificate)this.certs.remove(alias)) != null) {
         this.chainCerts.remove(new CertId(var3.getPublicKey()));
      }

      if (var2 != null) {
         String var4;
         if ((var4 = (String)this.localIds.remove(alias)) != null) {
            var3 = (Certificate)this.keyCerts.remove(var4);
         }

         if (var3 != null) {
            this.chainCerts.remove(new CertId(var3.getPublicKey()));
         }
      }

      if (var3 == null && var2 == null) {
         throw new KeyStoreException("no such entry as " + alias);
      }
   }

   public Certificate engineGetCertificate(String alias) {
      if (alias != null) {
         Certificate var2;
         if ((var2 = (Certificate)this.certs.get(alias)) == null) {
            String var3;
            if ((var3 = (String)this.localIds.get(alias)) != null) {
               var2 = (Certificate)this.keyCerts.get(var3);
            } else {
               var2 = (Certificate)this.keyCerts.get(alias);
            }
         }

         return var2;
      } else {
         throw new IllegalArgumentException("null alias passed to getCertificate.");
      }
   }

   public String engineGetCertificateAlias(Certificate cert) {
      Enumeration var2 = this.certs.elements();
      Enumeration var3 = this.certs.keys();

      Certificate var10000;
      while(var2.hasMoreElements()) {
         var10000 = (Certificate)var2.nextElement();
         String var4 = (String)var3.nextElement();
         if (var10000.equals(cert)) {
            return var4;
         }
      }

      Enumeration var7 = this.keyCerts.elements();
      Enumeration var5 = this.keyCerts.keys();

      while(var7.hasMoreElements()) {
         var10000 = (Certificate)var7.nextElement();
         String var6 = (String)var5.nextElement();
         if (var10000.equals(cert)) {
            return var6;
         }
      }

      return null;
   }

   public Certificate[] engineGetCertificateChain(String alias) {
      if (alias == null) {
         throw new IllegalArgumentException("null alias passed to getCertificateChain.");
      } else if (!this.engineIsKeyEntry(alias)) {
         return null;
      } else {
         Object alias1;
         if ((alias1 = this.engineGetCertificate(alias)) == null) {
            return null;
         } else {
            Vector var2;
            var2 = new Vector.<init>();

            while(alias1 != null) {
               X509Certificate var3;
               X509Certificate var10000 = var3 = (X509Certificate)alias1;
               Certificate var4 = null;
               byte[] var5;
               if ((var5 = var10000.getExtensionValue(X509Extensions.AuthorityKeyIdentifier.getId())) != null) {
                  label123: {
                     Certificate var27;
                     label89: {
                        IOException var22;
                        label118: {
                           boolean var10001;
                           byte[] var23;
                           try {
                              var23 = ((ASN1OctetString)(new ASN1InputStream(var5)).readObject()).getOctets();
                           } catch (IOException var12) {
                              var22 = var12;
                              var10001 = false;
                              break label118;
                           }

                           var5 = var23;

                           ASN1InputStream var24;
                           try {
                              var24 = new ASN1InputStream;
                           } catch (IOException var11) {
                              var22 = var11;
                              var10001 = false;
                              break label118;
                           }

                           ASN1InputStream var6 = var24;

                           AuthorityKeyIdentifier var25;
                           try {
                              var24.<init>(var5);
                              var25 = new AuthorityKeyIdentifier;
                           } catch (IOException var10) {
                              var22 = var10;
                              var10001 = false;
                              break label118;
                           }

                           AuthorityKeyIdentifier var19;
                           AuthorityKeyIdentifier var26 = var19 = var25;

                           try {
                              var26.<init>((ASN1Sequence)var6.readObject());
                              var23 = var25.getKeyIdentifier();
                           } catch (IOException var9) {
                              var22 = var9;
                              var10001 = false;
                              break label118;
                           }

                           if (var23 == null) {
                              break label123;
                           }

                           try {
                              var27 = (Certificate)this.chainCerts.get(new CertId(var19.getKeyIdentifier()));
                              break label89;
                           } catch (IOException var8) {
                              var22 = var8;
                              var10001 = false;
                           }
                        }

                        IOException var14 = var22;
                        throw new RuntimeException(var14.toString());
                     }

                     var4 = var27;
                  }
               }

               Object var7;
               label127: {
                  Principal var20;
                  if (var4 == null && !(var20 = var3.getIssuerDN()).equals(var3.getSubjectDN())) {
                     Enumeration var21 = this.chainCerts.keys();

                     label107:
                     while(true) {
                        do {
                           if (!var21.hasMoreElements()) {
                              break label107;
                           }
                        } while(!((X509Certificate)(var7 = (X509Certificate)this.chainCerts.get(var21.nextElement()))).getSubjectDN().equals(var20));

                        try {
                           var3.verify(((X509Certificate)var7).getPublicKey());
                           break label127;
                        } catch (Exception var13) {
                        }
                     }
                  }

                  var7 = var4;
               }

               var2.addElement(alias1);
               if (var7 != alias1) {
                  alias1 = var7;
               } else {
                  var7 = null;
                  alias1 = var7;
               }
            }

            int var17;
            Certificate[] var18 = new Certificate[var17 = var2.size()];

            for(int var15 = 0; var15 != var17; ++var15) {
               var18[var15] = (Certificate)var2.elementAt(var15);
            }

            return var18;
         }
      }
   }

   public Date engineGetCreationDate(String alias) {
      return new Date();
   }

   public Key engineGetKey(String alias, char[] password) {
      if (alias != null) {
         return (Key)this.keys.get(alias);
      } else {
         throw new IllegalArgumentException("null alias passed to getKey.");
      }
   }

   public boolean engineIsCertificateEntry(String alias) {
      return this.certs.get(alias) != null && this.keys.get(alias) == null;
   }

   public boolean engineIsKeyEntry(String alias) {
      return this.keys.get(alias) != null;
   }

   public void engineSetCertificateEntry(String alias, Certificate cert) {
      if (this.keys.get(alias) == null) {
         this.certs.put(alias, cert);
         this.chainCerts.put(new CertId(cert.getPublicKey()), cert);
      } else {
         throw new KeyStoreException("There is a key entry with the name " + alias + ".");
      }
   }

   public void engineSetKeyEntry(String alias, byte[] key, Certificate[] chain) {
      throw new RuntimeException("operation not supported");
   }

   public void engineSetKeyEntry(String alias, Key key, char[] password, Certificate[] chain) {
      if (key instanceof PrivateKey && chain == null) {
         throw new KeyStoreException("no certificate chain for private key");
      } else {
         if (this.keys.get(alias) != null) {
            this.engineDeleteEntry(alias);
         }

         this.keys.put(alias, key);
         this.certs.put(alias, chain[0]);

         for(int alias1 = 0; alias1 != chain.length; ++alias1) {
            Hashtable var10000 = this.chainCerts;
            CertId key1;
            key1 = new CertId.<init>(chain[alias1].getPublicKey());
            var10000.put(key1, chain[alias1]);
         }

      }
   }

   public int engineSize() {
      Hashtable var1;
      var1 = new Hashtable.<init>();
      Enumeration var2 = this.certs.keys();

      while(var2.hasMoreElements()) {
         var1.put(var2.nextElement(), "cert");
      }

      Enumeration var3 = this.keys.keys();

      while(var3.hasMoreElements()) {
         String var4;
         if (var1.get(var4 = (String)var3.nextElement()) == null) {
            var1.put(var4, "key");
         }
      }

      return var1.size();
   }

   protected PrivateKey unwrapKey(AlgorithmIdentifier algId, byte[] data, char[] password, boolean wrongPKCS12Zero) {
      String var10000 = algId.getObjectId().getId();
      String var10001 = var10000;
      PKCS12PBEParams var9;
      var9 = new PKCS12PBEParams.<init>((ASN1Sequence)algId.getParameters());
      PBEKeySpec algId1;
      algId1 = new PBEKeySpec.<init>(password);

      Exception var14;
      label41: {
         boolean var15;
         SecretKeyFactory var17;
         PBEKeySpec var10002;
         PBEParameterSpec var10003;
         try {
            var17 = SecretKeyFactory.getInstance(var10001, "INFOSEC");
            var10002 = algId1;
            var10003 = new PBEParameterSpec;
         } catch (Exception var8) {
            var14 = var8;
            var15 = false;
            break label41;
         }

         PBEParameterSpec algId2 = var10003;

         SecretKey var18;
         try {
            var10003.<init>(var9.getIV(), var9.getIterations().intValue());
            var18 = var17.generateSecret(var10002);
         } catch (Exception var7) {
            var14 = var7;
            var15 = false;
            break label41;
         }

         SecretKey var10 = var18;

         Cipher var16;
         try {
            ((JCEPBEKey)var18).setTryWrongPKCS12Zero(wrongPKCS12Zero);
            var16 = Cipher.getInstance(var10000, "INFOSEC");
         } catch (Exception var6) {
            var14 = var6;
            var15 = false;
            break label41;
         }

         Cipher var19 = var16;

         try {
            var19.init(4, var10, algId2);
            return (PrivateKey)var16.unwrap(data, "", 2);
         } catch (Exception var5) {
            var14 = var5;
            var15 = false;
         }
      }

      Exception var11 = var14;
      var11.printStackTrace();
      throw new IOException("exception unwrapping private key - " + var11.toString());
   }

   protected byte[] wrapKey(String algorithm, Key key, PKCS12PBEParams pbeParams, char[] password) {
      String var10000 = algorithm;
      String var10001 = algorithm;
      PBEKeySpec var9;
      var9 = new PBEKeySpec.<init>(password);

      Exception var14;
      label41: {
         boolean var15;
         SecretKeyFactory var17;
         try {
            var17 = SecretKeyFactory.getInstance(var10001, "INFOSEC");
         } catch (Exception var8) {
            var14 = var8;
            var15 = false;
            break label41;
         }

         SecretKeyFactory algorithm1 = var17;

         PBEParameterSpec var18;
         try {
            var18 = new PBEParameterSpec;
         } catch (Exception var7) {
            var14 = var7;
            var15 = false;
            break label41;
         }

         PBEParameterSpec password1 = var18;

         Cipher var16;
         try {
            var18.<init>(pbeParams.getIV(), pbeParams.getIterations().intValue());
            var16 = Cipher.getInstance(var10000, "INFOSEC");
         } catch (Exception var6) {
            var14 = var6;
            var15 = false;
            break label41;
         }

         Cipher var10002 = var16;
         Key var19 = key;
         SecretKeyFactory var10003 = algorithm1;
         PBEKeySpec var10004 = var9;
         byte var10 = 3;

         try {
            var10002.init(var10, var10003.generateSecret(var10004), password1);
            return var16.wrap(var19);
         } catch (Exception var5) {
            var14 = var5;
            var15 = false;
         }
      }

      Exception var11 = var14;
      throw new IOException("exception encrypting data - " + var11.toString());
   }

   protected byte[] cryptData(boolean forEncryption, AlgorithmIdentifier algId, char[] password, boolean wrongPKCS12Zero, byte[] data) {
      boolean var10000 = forEncryption;
      String var10001 = algId.getObjectId().getId();
      String var10002 = var10001;
      PKCS12PBEParams var10;
      var10 = new PKCS12PBEParams.<init>((ASN1Sequence)algId.getParameters());
      PBEKeySpec forEncryption1;
      forEncryption1 = new PBEKeySpec.<init>(password);

      Exception var17;
      label44: {
         boolean var18;
         SecretKeyFactory var19;
         PBEKeySpec var10003;
         PBEParameterSpec var10004;
         try {
            var19 = SecretKeyFactory.getInstance(var10002, "INFOSEC");
            var10003 = forEncryption1;
            var10004 = new PBEParameterSpec;
         } catch (Exception var9) {
            var17 = var9;
            var18 = false;
            break label44;
         }

         PBEParameterSpec forEncryption2 = var10004;

         JCEPBEKey var21;
         try {
            var10004.<init>(var10.getIV(), var10.getIterations().intValue());
            var21 = (JCEPBEKey)var19.generateSecret(var10003);
         } catch (Exception var8) {
            var17 = var8;
            var18 = false;
            break label44;
         }

         JCEPBEKey var11 = var21;

         Cipher var20;
         try {
            var21.setTryWrongPKCS12Zero(wrongPKCS12Zero);
            var20 = Cipher.getInstance(var10001, "INFOSEC");
         } catch (Exception var7) {
            var17 = var7;
            var18 = false;
            break label44;
         }

         Cipher algId1 = var20;
         byte password1;
         if (var10000) {
            password1 = 1;
         } else {
            password1 = 2;
         }

         try {
            algId1.init(password1, var11, forEncryption2);
            return algId1.doFinal(data);
         } catch (Exception var6) {
            var17 = var6;
            var18 = false;
         }
      }

      Exception var12 = var17;
      throw new IOException("exception decrypting data - " + var12.toString());
   }

   public void engineLoad(InputStream stream, char[] password) {
      if (stream != null) {
         if (password != null) {
            BufferedInputStream var3;
            BufferedInputStream var10000 = var3 = new BufferedInputStream;
            var10000.<init>(stream);
            var10000.mark(10);
            if (var10000.read() != 48) {
               throw new IOException("stream does not represent a PKCS12 key store");
            } else {
               var3.reset();
               ASN1Sequence stream1 = (ASN1Sequence)(new ASN1InputStream(var3)).readObject();
               Pfx var40;
               Pfx var67 = var40 = new Pfx;
               var67.<init>(stream1);
               ContentInfo stream2 = var67.getAuthSafe();
               Vector var4;
               var4 = new Vector.<init>();
               boolean var5 = false;
               boolean var6 = false;
               boolean var10001;
               Exception var31;
               int var43;
               Exception var70;
               if (var67.getMacData() != null) {
                  MacData var42;
                  MacData var68 = var42 = var40.getMacData();
                  DigestInfo var7;
                  AlgorithmIdentifier var8 = (var7 = var68.getMac()).getAlgorithmId();
                  byte[] var9 = var68.getSalt();
                  AlgorithmIdentifier var69 = var8;
                  var43 = var42.getIterationCount().intValue();
                  byte[] var10 = ((ASN1OctetString)stream2.getContent()).getOctets();

                  label287: {
                     byte[] var71;
                     IOException var72;
                     byte[] var73;
                     try {
                        var73 = calculatePbeMac(var69.getObjectId(), var9, var43, password, false, var10);
                        var71 = var7.getDigest();
                     } catch (IOException var29) {
                        var72 = var29;
                        var10001 = false;
                        throw var72;
                     } catch (Exception var30) {
                        var70 = var30;
                        var10001 = false;
                        break label287;
                     }

                     byte[] var47 = var71;

                     boolean var75;
                     try {
                        var75 = Arrays.areEqual(var73, var71);
                     } catch (IOException var27) {
                        var72 = var27;
                        var10001 = false;
                        throw var72;
                     } catch (Exception var28) {
                        var70 = var28;
                        var10001 = false;
                        break label287;
                     }

                     if (var75) {
                        break label121;
                     }

                     int var76;
                     try {
                        var76 = password.length;
                     } catch (IOException var25) {
                        var72 = var25;
                        var10001 = false;
                        throw var72;
                     } catch (Exception var26) {
                        var70 = var26;
                        var10001 = false;
                        break label287;
                     }

                     if (var76 <= 0) {
                        label309: {
                           try {
                              var75 = Arrays.areEqual(calculatePbeMac(var8.getObjectId(), var9, var43, password, true, var10), var47);
                           } catch (IOException var21) {
                              var72 = var21;
                              var10001 = false;
                              throw var72;
                           } catch (Exception var22) {
                              var70 = var22;
                              var10001 = false;
                              break label309;
                           }

                           if (var75) {
                              var6 = true;
                              break label121;
                           }

                           try {
                              throw new IOException("PKCS12 key store mac invalid - wrong password or corrupted file.");
                           } catch (IOException var19) {
                              var72 = var19;
                              var10001 = false;
                              throw var72;
                           } catch (Exception var20) {
                              var70 = var20;
                              var10001 = false;
                           }
                        }
                     } else {
                        try {
                           throw new IOException("PKCS12 key store mac invalid - wrong password or corrupted file.");
                        } catch (IOException var23) {
                           var72 = var23;
                           var10001 = false;
                           throw var72;
                        } catch (Exception var24) {
                           var70 = var24;
                           var10001 = false;
                        }
                     }
                  }

                  var31 = var70;
                  var31.printStackTrace();
                  throw new IOException("error constructing MAC: " + var31.toString());
               }

               this.keys = new IgnoresCaseHashtable((IgnoresCaseHashtable)null);
               this.localIds = new Hashtable();
               SafeBag var81;
               if (stream2.getContentType().equals(PKCSObjectIdentifiers.data)) {
                  ASN1InputStream var44;
                  var44 = new ASN1InputStream.<init>(((ASN1OctetString)stream2.getContent()).getOctets());
                  ContentInfo[] stream3 = (new AuthenticatedSafe((ASN1Sequence)var44.readObject())).getContentInfo();

                  for(var43 = 0; var43 != stream3.length; ++var43) {
                     PKCS12BagAttributeCarrier var11;
                     DERObjectIdentifier var14;
                     ASN1Set var15;
                     DERObject var16;
                     EncryptedPrivateKeyInfo var10002;
                     ASN1Sequence var51;
                     int var54;
                     SafeBag var55;
                     String var57;
                     PrivateKey var60;
                     ASN1Sequence var83;
                     ASN1Set var84;
                     if (stream3[var43].getContentType().equals(PKCSObjectIdentifiers.data)) {
                        var51 = (ASN1Sequence)(new ASN1InputStream(((ASN1OctetString)stream3[var43].getContent()).getOctets())).readObject();

                        for(var54 = 0; var54 != var51.size(); ++var54) {
                           var81 = var55 = new SafeBag;
                           var81.<init>((ASN1Sequence)var51.getObjectAt(var54));
                           if (!var81.getBagId().equals(PKCSObjectIdentifiers.pkcs8ShroudedKeyBag)) {
                              if (var55.getBagId().equals(PKCSObjectIdentifiers.certBag)) {
                                 var4.addElement(var55);
                              } else {
                                 System.out.println("extra in data " + var55.getBagId());
                                 System.out.println(ASN1Dump.dumpAsString(var55));
                              }
                           } else {
                              var10002 = new EncryptedPrivateKeyInfo((ASN1Sequence)var55.getBagValue());
                              AlgorithmIdentifier var61 = var10002.getEncryptionAlgorithm();
                              var11 = (PKCS12BagAttributeCarrier)(var60 = this.unwrapKey(var61, var10002.getEncryptedData(), password, var6));
                              String var65 = null;
                              ASN1OctetString var66 = null;
                              if (var55.getBagAttributes() != null) {
                                 Enumeration var63 = var55.getBagAttributes().getObjects();

                                 while(var63.hasMoreElements()) {
                                    var83 = (ASN1Sequence)var63.nextElement();
                                    var14 = (DERObjectIdentifier)var83.getObjectAt(0);
                                    var84 = var15 = (ASN1Set)var83.getObjectAt(1);
                                    var16 = null;
                                    if (var84.size() > 0) {
                                       var11.setBagAttribute(var14, var16 = (DERObject)var15.getObjectAt(0));
                                    }

                                    if (var14.equals(PKCSObjectIdentifiers.pkcs_9_at_friendlyName)) {
                                       var65 = ((DERBMPString)var16).getString();
                                       this.keys.put(var65, var60);
                                    } else if (var14.equals(PKCSObjectIdentifiers.pkcs_9_at_localKeyId)) {
                                       var66 = (ASN1OctetString)var16;
                                    }
                                 }
                              }

                              if (var66 != null) {
                                 var57 = new String.<init>(Hex.encode(var66.getOctets()));
                                 if (var65 == null) {
                                    this.keys.put(var57, var60);
                                 } else {
                                    this.localIds.put(var65, var57);
                                 }
                              } else {
                                 var5 = true;
                                 this.keys.put("unmarked", var60);
                              }
                           }
                        }
                     } else if (!stream3[var43].getContentType().equals(PKCSObjectIdentifiers.encryptedData)) {
                        PrintStream var85 = System.out;
                        StringBuffer var52;
                        StringBuffer var78 = var52 = new StringBuffer;
                        var52.<init>("extra ");
                        var85.println(var78.append(stream3[var43].getContentType().getId()).toString());
                        var85 = System.out;
                        var78 = var52 = new StringBuffer;
                        var52.<init>("extra ");
                        var85.println(var78.append(ASN1Dump.dumpAsString(stream3[var43].getContent())).toString());
                     } else {
                        EncryptedData var77 = new EncryptedData((ASN1Sequence)stream3[var43].getContent());
                        AlgorithmIdentifier var49 = var77.getEncryptionAlgorithm();
                        byte[] var53 = var77.getContent().getOctets();
                        var51 = (ASN1Sequence)ASN1Object.fromByteArray(this.cryptData(false, var49, password, var6, var53));

                        for(var54 = 0; var54 != var51.size(); ++var54) {
                           var81 = var55 = new SafeBag;
                           var81.<init>((ASN1Sequence)var51.getObjectAt(var54));
                           if (var81.getBagId().equals(PKCSObjectIdentifiers.certBag)) {
                              var4.addElement(var55);
                           } else {
                              ASN1OctetString var12;
                              Enumeration var13;
                              String var62;
                              if (var55.getBagId().equals(PKCSObjectIdentifiers.pkcs8ShroudedKeyBag)) {
                                 var10002 = new EncryptedPrivateKeyInfo((ASN1Sequence)var55.getBagValue());
                                 AlgorithmIdentifier var59 = var10002.getEncryptionAlgorithm();
                                 var11 = (PKCS12BagAttributeCarrier)(var60 = this.unwrapKey(var59, var10002.getEncryptedData(), password, var6));
                                 var57 = null;
                                 var12 = null;
                                 var13 = var55.getBagAttributes().getObjects();

                                 while(var13.hasMoreElements()) {
                                    var83 = (ASN1Sequence)var13.nextElement();
                                    var14 = (DERObjectIdentifier)var83.getObjectAt(0);
                                    var84 = var15 = (ASN1Set)var83.getObjectAt(1);
                                    var16 = null;
                                    if (var84.size() > 0) {
                                       var11.setBagAttribute(var14, var16 = (DERObject)var15.getObjectAt(0));
                                    }

                                    if (var14.equals(PKCSObjectIdentifiers.pkcs_9_at_friendlyName)) {
                                       var57 = ((DERBMPString)var16).getString();
                                       this.keys.put(var57, var60);
                                    } else if (var14.equals(PKCSObjectIdentifiers.pkcs_9_at_localKeyId)) {
                                       var12 = (ASN1OctetString)var16;
                                    }
                                 }

                                 var62 = new String.<init>(Hex.encode(var12.getOctets()));
                                 if (var57 == null) {
                                    this.keys.put(var62, var60);
                                 } else {
                                    this.localIds.put(var57, var62);
                                 }
                              } else if (!var55.getBagId().equals(PKCSObjectIdentifiers.keyBag)) {
                                 System.out.println("extra in encryptedData " + var55.getBagId());
                                 System.out.println(ASN1Dump.dumpAsString(var55));
                              } else {
                                 var11 = (PKCS12BagAttributeCarrier)(var60 = JDKKeyFactory.createPrivateKeyFromPrivateKeyInfo(new PrivateKeyInfo((ASN1Sequence)var55.getBagValue())));
                                 var57 = null;
                                 var12 = null;
                                 var13 = var55.getBagAttributes().getObjects();

                                 while(var13.hasMoreElements()) {
                                    var83 = (ASN1Sequence)var13.nextElement();
                                    var14 = (DERObjectIdentifier)var83.getObjectAt(0);
                                    var84 = var15 = (ASN1Set)var83.getObjectAt(1);
                                    var16 = null;
                                    if (var84.size() > 0) {
                                       var11.setBagAttribute(var14, var16 = (DERObject)var15.getObjectAt(0));
                                    }

                                    if (var14.equals(PKCSObjectIdentifiers.pkcs_9_at_friendlyName)) {
                                       var57 = ((DERBMPString)var16).getString();
                                       this.keys.put(var57, var60);
                                    } else if (var14.equals(PKCSObjectIdentifiers.pkcs_9_at_localKeyId)) {
                                       var12 = (ASN1OctetString)var16;
                                    }
                                 }

                                 var62 = new String.<init>(Hex.encode(var12.getOctets()));
                                 if (var57 == null) {
                                    this.keys.put(var62, var60);
                                 } else {
                                    this.localIds.put(var57, var62);
                                 }
                              }
                           }
                        }
                     }
                  }
               }

               IgnoresCaseHashtable stream4;
               stream4 = new IgnoresCaseHashtable.<init>((IgnoresCaseHashtable)null);
               this.certs = stream4;
               Hashtable stream5;
               stream5 = new Hashtable.<init>();
               this.chainCerts = stream5;
               stream5 = new Hashtable.<init>();
               this.keyCerts = stream5;

               for(int stream6 = 0; stream6 != var4.size(); ++stream6) {
                  SafeBag password1 = (SafeBag)var4.elementAt(stream6);
                  CertBag var45;
                  CertBag var86 = var45 = new CertBag;
                  var86.<init>((ASN1Sequence)password1.getBagValue());
                  if (!var86.getCertId().equals(PKCSObjectIdentifiers.x509Certificate)) {
                     throw new RuntimeException("Unsupported certificate type: " + var45.getCertId());
                  }

                  Certificate var80;
                  label188: {
                     label297: {
                        ByteArrayInputStream var74;
                        JDKPKCS12KeyStore var79;
                        try {
                           var81 = password1;
                           var79 = this;
                           var74 = new ByteArrayInputStream;
                        } catch (Exception var18) {
                           var70 = var18;
                           var10001 = false;
                           break label297;
                        }

                        ByteArrayInputStream var48 = var74;

                        try {
                           var74.<init>(((ASN1OctetString)var45.getCertValue()).getOctets());
                           var80 = var79.certFact.generateCertificate(var48);
                           break label188;
                        } catch (Exception var17) {
                           var70 = var17;
                           var10001 = false;
                        }
                     }

                     var31 = var70;
                     throw new RuntimeException(var31.toString());
                  }

                  Certificate var46 = var80;
                  ASN1OctetString var50 = null;
                  String var56 = null;
                  if (var81.getBagAttributes() != null) {
                     Enumeration password2 = password1.getBagAttributes().getObjects();

                     while(password2.hasMoreElements()) {
                        ASN1Sequence var82 = (ASN1Sequence)password2.nextElement();
                        DERObjectIdentifier var58 = (DERObjectIdentifier)var82.getObjectAt(0);
                        DERObject var64 = (DERObject)((ASN1Set)var82.getObjectAt(1)).getObjectAt(0);
                        if (var46 instanceof PKCS12BagAttributeCarrier) {
                           ((PKCS12BagAttributeCarrier)var46).setBagAttribute(var58, var64);
                        }

                        if (var58.equals(PKCSObjectIdentifiers.pkcs_9_at_friendlyName)) {
                           var56 = ((DERBMPString)var64).getString();
                        } else if (var58.equals(PKCSObjectIdentifiers.pkcs_9_at_localKeyId)) {
                           var50 = (ASN1OctetString)var64;
                        }
                     }
                  }

                  this.chainCerts.put(new CertId(var46.getPublicKey()), var46);
                  String password3;
                  if (var5) {
                     if (this.keyCerts.isEmpty()) {
                        password3 = new String.<init>(Hex.encode(this.createSubjectKeyId(var46.getPublicKey()).getKeyIdentifier()));
                        this.keyCerts.put(password3, var46);
                        IgnoresCaseHashtable var87 = this.keys;
                        var87.put(password3, var87.remove("unmarked"));
                     }
                  } else {
                     if (var50 != null) {
                        password3 = new String.<init>(Hex.encode(var50.getOctets()));
                        this.keyCerts.put(password3, var46);
                     }

                     if (var56 != null) {
                        this.certs.put(var56, var46);
                     }
                  }
               }

            }
         } else {
            throw new NullPointerException("No password supplied for PKCS#12 KeyStore.");
         }
      }
   }

   public void engineStore(OutputStream stream, char[] password) {
      if (password == null) {
         throw new NullPointerException("No password supplied for PKCS#12 KeyStore.");
      } else {
         ASN1EncodableVector var3;
         var3 = new ASN1EncodableVector.<init>();
         Enumeration var4 = this.keys.keys();

         byte[] var5;
         ASN1EncodableVector var10001;
         ASN1EncodableVector var10002;
         DERObjectIdentifier var92;
         DERObjectIdentifier var109;
         Certificate var111;
         DERObjectIdentifier var115;
         SafeBag var131;
         while(var4.hasMoreElements()) {
            var5 = new byte[20];
            this.random.nextBytes(var5);
            String var6 = (String)var4.nextElement();
            PrivateKey var7 = (PrivateKey)this.keys.get(var6);
            PKCS12PBEParams var8;
            var8 = new PKCS12PBEParams.<init>(var5, 1024);
            byte[] var9 = this.wrapKey((var92 = KEY_ALGORITHM).getId(), var7, var8, password);
            AlgorithmIdentifier var10;
            var10 = new AlgorithmIdentifier.<init>(var92, var8.getDERObject());
            EncryptedPrivateKeyInfo var94;
            var94 = new EncryptedPrivateKeyInfo.<init>(var10, var9);
            boolean var104 = false;
            ASN1EncodableVector var110;
            var110 = new ASN1EncodableVector.<init>();
            if (var7 instanceof PKCS12BagAttributeCarrier) {
               DERBMPString var11;
               PKCS12BagAttributeCarrier var100;
               if ((var11 = (DERBMPString)(var100 = (PKCS12BagAttributeCarrier)var7).getBagAttribute(var115 = PKCSObjectIdentifiers.pkcs_9_at_friendlyName)) == null || !var11.getString().equals(var6)) {
                  var11 = new DERBMPString.<init>(var6);
                  var100.setBagAttribute(var115, var11);
               }

               if (var100.getBagAttribute(var115 = PKCSObjectIdentifiers.pkcs_9_at_localKeyId) == null) {
                  var100.setBagAttribute(var115, this.createSubjectKeyId(this.engineGetCertificate(var6).getPublicKey()));
               }

               Enumeration var117 = var100.getBagAttributeKeys();

               while(var117.hasMoreElements()) {
                  var109 = (DERObjectIdentifier)var117.nextElement();
                  ASN1EncodableVector var120;
                  var10001 = var120 = new ASN1EncodableVector;
                  var120.<init>();
                  var10001.add(var109);
                  var10001.add(new DERSet(var100.getBagAttribute(var109)));
                  var104 = true;
                  var110.add(new DERSequence(var120));
               }
            }

            if (!var104) {
               ASN1EncodableVector var102;
               var10002 = var102 = new ASN1EncodableVector;
               var102.<init>();
               var111 = this.engineGetCertificate(var6);
               var10002.add(PKCSObjectIdentifiers.pkcs_9_at_localKeyId);
               var10002.add(new DERSet(this.createSubjectKeyId(var111.getPublicKey())));
               var110.add(new DERSequence(var102));
               var10001 = var102 = new ASN1EncodableVector;
               var10001.<init>();
               var10001.add(PKCSObjectIdentifiers.pkcs_9_at_friendlyName);
               var10001.add(new DERSet(new DERBMPString(var6)));
               var110.add(new DERSequence(var102));
            }

            SafeBag var96;
            var131 = var96 = new SafeBag;
            EncryptedPrivateKeyInfo var134 = var94;
            var92 = PKCSObjectIdentifiers.pkcs8ShroudedKeyBag;
            DERObject var137 = var134.getDERObject();
            DERSet var105;
            var105 = new DERSet.<init>(var110);
            var131.<init>(var92, var137, var105);
            var3.add(var96);
         }

         byte[] var85 = (new DERSequence(var3)).getDEREncoded();
         BERConstructedOctetString var87;
         var87 = new BERConstructedOctetString.<init>(var85);
         var85 = new byte[20];
         this.random.nextBytes(var85);
         ASN1EncodableVector var95;
         var95 = new ASN1EncodableVector.<init>();
         PKCS12PBEParams var97;
         var97 = new PKCS12PBEParams.<init>(var85, 1024);
         AlgorithmIdentifier var86;
         AlgorithmIdentifier var132 = var86 = new AlgorithmIdentifier;
         PKCS12PBEParams var138 = var97;
         DERObjectIdentifier var98 = CERT_ALGORITHM;
         var132.<init>(var98, var138.getDERObject());
         Hashtable var99;
         var99 = new Hashtable.<init>();
         Enumeration var108 = this.keys.keys();

         ASN1EncodableVector var12;
         PKCS12BagAttributeCarrier var13;
         DERObjectIdentifier var14;
         DERBMPString var15;
         CertificateEncodingException var10000;
         ASN1EncodableVector var10003;
         SafeBag var10005;
         CertBag var10006;
         DERObjectIdentifier var10007;
         CertificateEncodingException var79;
         String var112;
         Certificate var113;
         boolean var119;
         CertBag var122;
         ASN1EncodableVector var123;
         Enumeration var127;
         ASN1EncodableVector var128;
         JDKPKCS12KeyStore var129;
         Certificate var130;
         boolean var133;
         boolean var135;
         PKCS12BagAttributeCarrier var139;
         CertBag var140;
         DERBMPString var141;
         Certificate var142;
         SafeBag var143;
         DERObjectIdentifier var147;
         Enumeration var151;
         ASN1EncodableVector var152;
         Hashtable var153;
         Certificate var154;
         while(var108.hasMoreElements()) {
            label563: {
               String var136;
               try {
                  var129 = this;
                  var136 = (String)var108.nextElement();
               } catch (CertificateEncodingException var78) {
                  var10000 = var78;
                  var133 = false;
                  break label563;
               }

               var112 = var136;

               try {
                  var130 = var129.engineGetCertificate(var136);
               } catch (CertificateEncodingException var77) {
                  var10000 = var77;
                  var133 = false;
                  break label563;
               }

               var113 = var130;
               var119 = false;

               try {
                  var140 = new CertBag;
               } catch (CertificateEncodingException var76) {
                  var10000 = var76;
                  var133 = false;
                  break label563;
               }

               var122 = var140;

               try {
                  var140.<init>(PKCSObjectIdentifiers.x509Certificate, new DEROctetString(var113.getEncoded()));
                  var10001 = new ASN1EncodableVector;
               } catch (CertificateEncodingException var75) {
                  var10000 = var75;
                  var133 = false;
                  break label563;
               }

               var12 = var10001;

               try {
                  var10001.<init>();
                  var135 = var130 instanceof PKCS12BagAttributeCarrier;
               } catch (CertificateEncodingException var74) {
                  var10000 = var74;
                  var133 = false;
                  break label563;
               }

               if (var135) {
                  try {
                     var139 = (PKCS12BagAttributeCarrier)var113;
                  } catch (CertificateEncodingException var73) {
                     var10000 = var73;
                     var133 = false;
                     break label563;
                  }

                  var13 = var139;

                  try {
                     var147 = PKCSObjectIdentifiers.pkcs_9_at_friendlyName;
                  } catch (CertificateEncodingException var72) {
                     var10000 = var72;
                     var133 = false;
                     break label563;
                  }

                  var14 = var147;

                  try {
                     var141 = (DERBMPString)var139.getBagAttribute(var147);
                  } catch (CertificateEncodingException var71) {
                     var10000 = var71;
                     var133 = false;
                     break label563;
                  }

                  label511: {
                     var15 = var141;
                     if (var141 != null) {
                        try {
                           var135 = var15.getString().equals(var112);
                        } catch (CertificateEncodingException var70) {
                           var10000 = var70;
                           var133 = false;
                           break label563;
                        }

                        if (var135) {
                           break label511;
                        }
                     }

                     try {
                        var13.setBagAttribute(var14, new DERBMPString(var112));
                     } catch (CertificateEncodingException var69) {
                        var10000 = var69;
                        var133 = false;
                        break label563;
                     }
                  }

                  try {
                     var139 = var13;
                     var147 = PKCSObjectIdentifiers.pkcs_9_at_localKeyId;
                  } catch (CertificateEncodingException var68) {
                     var10000 = var68;
                     var133 = false;
                     break label563;
                  }

                  var14 = var147;

                  DEREncodable var149;
                  try {
                     var149 = var139.getBagAttribute(var147);
                  } catch (CertificateEncodingException var67) {
                     var10000 = var67;
                     var133 = false;
                     break label563;
                  }

                  if (var149 == null) {
                     try {
                        var13.setBagAttribute(var14, this.createSubjectKeyId(var113.getPublicKey()));
                     } catch (CertificateEncodingException var66) {
                        var10000 = var66;
                        var133 = false;
                        break label563;
                     }
                  }

                  try {
                     var151 = var13.getBagAttributeKeys();
                  } catch (CertificateEncodingException var65) {
                     var10000 = var65;
                     var133 = false;
                     break label563;
                  }

                  var127 = var151;

                  while(true) {
                     try {
                        var135 = var127.hasMoreElements();
                     } catch (CertificateEncodingException var61) {
                        var10000 = var61;
                        var133 = false;
                        break label563;
                     }

                     if (!var135) {
                        break;
                     }

                     try {
                        var152 = var12;
                        var147 = (DERObjectIdentifier)var127.nextElement();
                     } catch (CertificateEncodingException var64) {
                        var10000 = var64;
                        var133 = false;
                        break label563;
                     }

                     var115 = var147;

                     try {
                        var10001 = new ASN1EncodableVector;
                     } catch (CertificateEncodingException var63) {
                        var10000 = var63;
                        var133 = false;
                        break label563;
                     }

                     var10002 = var128 = var10001;

                     try {
                        var128.<init>();
                        var10002.add(var115);
                        var10001.add(new DERSet(var13.getBagAttribute(var115)));
                        var152.add(new DERSequence(var128));
                     } catch (CertificateEncodingException var62) {
                        var10000 = var62;
                        var133 = false;
                        break label563;
                     }

                     var119 = true;
                  }
               }

               if (!var119) {
                  try {
                     var152 = var12;
                     var10001 = var12;
                     var10002 = new ASN1EncodableVector;
                  } catch (CertificateEncodingException var60) {
                     var10000 = var60;
                     var133 = false;
                     break label563;
                  }

                  var10003 = var10002;
                  ASN1EncodableVector var10004 = var123 = var10002;

                  try {
                     var10004.<init>();
                     var10003.add(PKCSObjectIdentifiers.pkcs_9_at_localKeyId);
                     var10002.add(new DERSet(this.createSubjectKeyId(var113.getPublicKey())));
                     var10001.add(new DERSequence(var123));
                     var10001 = new ASN1EncodableVector;
                  } catch (CertificateEncodingException var59) {
                     var10000 = var59;
                     var133 = false;
                     break label563;
                  }

                  var10002 = var10001;
                  var10003 = var123 = var10001;

                  try {
                     var10003.<init>();
                     var10002.add(PKCSObjectIdentifiers.pkcs_9_at_friendlyName);
                     var10001.add(new DERSet(new DERBMPString(var112)));
                     var152.add(new DERSequence(var123));
                  } catch (CertificateEncodingException var58) {
                     var10000 = var58;
                     var133 = false;
                     break label563;
                  }
               }

               try {
                  var153 = var99;
                  var154 = var113;
                  var142 = var113;
                  var10003 = var95;
                  var143 = new SafeBag;
                  var10005 = var143;
                  var10006 = var122;
                  var10007 = PKCSObjectIdentifiers.certBag;
               } catch (CertificateEncodingException var57) {
                  var10000 = var57;
                  var133 = false;
                  break label563;
               }

               var109 = var10007;

               try {
                  var10005.<init>(var109, var10006.getDERObject(), new DERSet(var12));
                  var10003.add(var143);
                  var153.put(var154, var142);
                  continue;
               } catch (CertificateEncodingException var56) {
                  var10000 = var56;
                  var133 = false;
               }
            }

            var79 = var10000;
            throw new IOException("Error encoding certificate: " + var79.toString());
         }

         var108 = this.certs.keys();

         JDKPKCS12KeyStore var155;
         Object var156;
         while(var108.hasMoreElements()) {
            label568: {
               String var144;
               try {
                  var129 = this;
                  var155 = this;
                  var144 = (String)var108.nextElement();
               } catch (CertificateEncodingException var55) {
                  var10000 = var55;
                  var133 = false;
                  break label568;
               }

               var112 = var144;

               try {
                  var154 = (Certificate)var155.certs.get(var112);
               } catch (CertificateEncodingException var54) {
                  var10000 = var54;
                  var133 = false;
                  break label568;
               }

               var113 = var154;
               var119 = false;

               try {
                  var156 = var129.keys.get(var112);
               } catch (CertificateEncodingException var53) {
                  var10000 = var53;
                  var133 = false;
                  break label568;
               }

               if (var156 != null) {
                  continue;
               }

               try {
                  var130 = var113;
                  var140 = new CertBag;
               } catch (CertificateEncodingException var52) {
                  var10000 = var52;
                  var133 = false;
                  break label568;
               }

               var122 = var140;

               try {
                  var140.<init>(PKCSObjectIdentifiers.x509Certificate, new DEROctetString(var113.getEncoded()));
                  var10001 = new ASN1EncodableVector;
               } catch (CertificateEncodingException var51) {
                  var10000 = var51;
                  var133 = false;
                  break label568;
               }

               var12 = var10001;

               try {
                  var10001.<init>();
                  var135 = var130 instanceof PKCS12BagAttributeCarrier;
               } catch (CertificateEncodingException var50) {
                  var10000 = var50;
                  var133 = false;
                  break label568;
               }

               if (var135) {
                  try {
                     var139 = (PKCS12BagAttributeCarrier)var113;
                  } catch (CertificateEncodingException var49) {
                     var10000 = var49;
                     var133 = false;
                     break label568;
                  }

                  var13 = var139;

                  try {
                     var147 = PKCSObjectIdentifiers.pkcs_9_at_friendlyName;
                  } catch (CertificateEncodingException var48) {
                     var10000 = var48;
                     var133 = false;
                     break label568;
                  }

                  var14 = var147;

                  try {
                     var141 = (DERBMPString)var139.getBagAttribute(var147);
                  } catch (CertificateEncodingException var47) {
                     var10000 = var47;
                     var133 = false;
                     break label568;
                  }

                  label419: {
                     var15 = var141;
                     if (var141 != null) {
                        try {
                           var135 = var15.getString().equals(var112);
                        } catch (CertificateEncodingException var46) {
                           var10000 = var46;
                           var133 = false;
                           break label568;
                        }

                        if (var135) {
                           break label419;
                        }
                     }

                     try {
                        var13.setBagAttribute(var14, new DERBMPString(var112));
                     } catch (CertificateEncodingException var45) {
                        var10000 = var45;
                        var133 = false;
                        break label568;
                     }
                  }

                  try {
                     var151 = var13.getBagAttributeKeys();
                  } catch (CertificateEncodingException var44) {
                     var10000 = var44;
                     var133 = false;
                     break label568;
                  }

                  var127 = var151;

                  while(true) {
                     try {
                        var135 = var127.hasMoreElements();
                     } catch (CertificateEncodingException var40) {
                        var10000 = var40;
                        var133 = false;
                        break label568;
                     }

                     if (!var135) {
                        break;
                     }

                     try {
                        var152 = var12;
                        var147 = (DERObjectIdentifier)var127.nextElement();
                     } catch (CertificateEncodingException var43) {
                        var10000 = var43;
                        var133 = false;
                        break label568;
                     }

                     var115 = var147;

                     try {
                        var10001 = new ASN1EncodableVector;
                     } catch (CertificateEncodingException var42) {
                        var10000 = var42;
                        var133 = false;
                        break label568;
                     }

                     var10002 = var128 = var10001;

                     try {
                        var128.<init>();
                        var10002.add(var115);
                        var10001.add(new DERSet(var13.getBagAttribute(var115)));
                        var152.add(new DERSequence(var128));
                     } catch (CertificateEncodingException var41) {
                        var10000 = var41;
                        var133 = false;
                        break label568;
                     }

                     var119 = true;
                  }
               }

               if (!var119) {
                  try {
                     var152 = var12;
                     var10001 = new ASN1EncodableVector;
                  } catch (CertificateEncodingException var39) {
                     var10000 = var39;
                     var133 = false;
                     break label568;
                  }

                  var10002 = var10001;
                  var10003 = var123 = var10001;

                  try {
                     var10003.<init>();
                     var10002.add(PKCSObjectIdentifiers.pkcs_9_at_friendlyName);
                     var10001.add(new DERSet(new DERBMPString(var112)));
                     var152.add(new DERSequence(var123));
                  } catch (CertificateEncodingException var38) {
                     var10000 = var38;
                     var133 = false;
                     break label568;
                  }
               }

               try {
                  var153 = var99;
                  var154 = var113;
                  var142 = var113;
                  var10003 = var95;
                  var143 = new SafeBag;
                  var10005 = var143;
                  var10006 = var122;
                  var10007 = PKCSObjectIdentifiers.certBag;
               } catch (CertificateEncodingException var37) {
                  var10000 = var37;
                  var133 = false;
                  break label568;
               }

               var109 = var10007;

               try {
                  var10005.<init>(var109, var10006.getDERObject(), new DERSet(var12));
                  var10003.add(var143);
                  var153.put(var154, var142);
                  continue;
               } catch (CertificateEncodingException var36) {
                  var10000 = var36;
                  var133 = false;
               }
            }

            var79 = var10000;
            throw new IOException("Error encoding certificate: " + var79.toString());
         }

         var108 = this.chainCerts.keys();

         while(var108.hasMoreElements()) {
            label573: {
               CertId var148;
               try {
                  var153 = var99;
                  var155 = this;
                  var148 = (CertId)var108.nextElement();
               } catch (CertificateEncodingException var35) {
                  var10000 = var35;
                  var133 = false;
                  break label573;
               }

               CertId var118 = var148;

               try {
                  var154 = (Certificate)var155.chainCerts.get(var118);
               } catch (CertificateEncodingException var34) {
                  var10000 = var34;
                  var133 = false;
                  break label573;
               }

               var111 = var154;

               try {
                  var156 = var153.get(var154);
               } catch (CertificateEncodingException var33) {
                  var10000 = var33;
                  var133 = false;
                  break label573;
               }

               if (var156 != null) {
                  continue;
               }

               try {
                  var130 = var111;
                  var140 = new CertBag;
               } catch (CertificateEncodingException var32) {
                  var10000 = var32;
                  var133 = false;
                  break label573;
               }

               CertBag var116 = var140;

               try {
                  var140.<init>(PKCSObjectIdentifiers.x509Certificate, new DEROctetString(var111.getEncoded()));
                  var10001 = new ASN1EncodableVector;
               } catch (CertificateEncodingException var31) {
                  var10000 = var31;
                  var133 = false;
                  break label573;
               }

               var123 = var10001;

               try {
                  var10001.<init>();
                  var135 = var130 instanceof PKCS12BagAttributeCarrier;
               } catch (CertificateEncodingException var30) {
                  var10000 = var30;
                  var133 = false;
                  break label573;
               }

               if (var135) {
                  try {
                     var139 = (PKCS12BagAttributeCarrier)var111;
                  } catch (CertificateEncodingException var29) {
                     var10000 = var29;
                     var133 = false;
                     break label573;
                  }

                  PKCS12BagAttributeCarrier var121 = var139;

                  try {
                     var151 = var139.getBagAttributeKeys();
                  } catch (CertificateEncodingException var28) {
                     var10000 = var28;
                     var133 = false;
                     break label573;
                  }

                  Enumeration var124 = var151;

                  while(true) {
                     try {
                        var135 = var124.hasMoreElements();
                     } catch (CertificateEncodingException var24) {
                        var10000 = var24;
                        var133 = false;
                        break label573;
                     }

                     if (!var135) {
                        break;
                     }

                     try {
                        var152 = var123;
                        var147 = (DERObjectIdentifier)var124.nextElement();
                     } catch (CertificateEncodingException var27) {
                        var10000 = var27;
                        var133 = false;
                        break label573;
                     }

                     DERObjectIdentifier var125 = var147;

                     try {
                        var10001 = new ASN1EncodableVector;
                     } catch (CertificateEncodingException var26) {
                        var10000 = var26;
                        var133 = false;
                        break label573;
                     }

                     ASN1EncodableVector var126;
                     var10002 = var126 = var10001;

                     try {
                        var126.<init>();
                        var10002.add(var125);
                        var10001.add(new DERSet(var121.getBagAttribute(var125)));
                        var152.add(new DERSequence(var126));
                     } catch (CertificateEncodingException var25) {
                        var10000 = var25;
                        var133 = false;
                        break label573;
                     }
                  }
               }

               CertBag var145;
               DERObjectIdentifier var146;
               SafeBag var150;
               try {
                  var152 = var95;
                  var131 = new SafeBag;
                  var150 = var131;
                  var145 = var116;
                  var146 = PKCSObjectIdentifiers.certBag;
               } catch (CertificateEncodingException var23) {
                  var10000 = var23;
                  var133 = false;
                  break label573;
               }

               var109 = var146;

               try {
                  var150.<init>(var109, var145.getDERObject(), new DERSet(var123));
                  var152.add(var131);
                  continue;
               } catch (CertificateEncodingException var22) {
                  var10000 = var22;
                  var133 = false;
               }
            }

            var79 = var10000;
            throw new IOException("Error encoding certificate: " + var79.toString());
         }

         var5 = (new DERSequence(var95)).getDEREncoded();
         var5 = this.cryptData(true, var86, password, false, var5);
         EncryptedData var103;
         DERObjectIdentifier var114;
         var103 = new EncryptedData.<init>(var114 = PKCSObjectIdentifiers.data, var86, new BERConstructedOctetString(var5));
         ContentInfo[] var88;
         ContentInfo[] var158 = var88 = new ContentInfo[2];
         ContentInfo var101;
         var101 = new ContentInfo.<init>(var114, var87);
         var158[0] = var101;
         ContentInfo var89;
         ContentInfo var157 = var89 = new ContentInfo;
         var92 = PKCSObjectIdentifiers.encryptedData;
         var157.<init>(var92, var103.getDERObject());
         var158[1] = var89;
         AuthenticatedSafe var91;
         var91 = new AuthenticatedSafe.<init>(var88);
         ByteArrayOutputStream var90;
         ByteArrayOutputStream var159 = var90 = new ByteArrayOutputStream;
         var159.<init>();
         (new BEROutputStream(var90)).writeObject(var91);
         var85 = var159.toByteArray();
         ContentInfo var160 = var89 = new ContentInfo;
         var155 = this;
         var89.<init>(var114, new BERConstructedOctetString(var85));
         byte[] var80 = new byte[20];
         short var93 = 1024;
         var155.random.nextBytes(var80);
         var5 = ((ASN1OctetString)var160.getContent()).getOctets();

         MacData password2;
         label308: {
            Exception var161;
            label576: {
               DERObjectIdentifier var162;
               try {
                  var162 = X509ObjectIdentifiers.id_SHA1;
               } catch (Exception var21) {
                  var161 = var21;
                  var133 = false;
                  break label576;
               }

               var98 = var162;

               byte[] var163;
               try {
                  var163 = calculatePbeMac(var162, var80, 1024, password, false, var5);
               } catch (Exception var20) {
                  var161 = var20;
                  var133 = false;
                  break label576;
               }

               byte[] password1 = var163;

               AlgorithmIdentifier var164;
               try {
                  var164 = new AlgorithmIdentifier;
               } catch (Exception var19) {
                  var161 = var19;
                  var133 = false;
                  break label576;
               }

               AlgorithmIdentifier var106 = var164;

               DigestInfo var165;
               try {
                  var164.<init>(var98, new DERNull());
                  var165 = new DigestInfo;
               } catch (Exception var18) {
                  var161 = var18;
                  var133 = false;
                  break label576;
               }

               DigestInfo var107 = var165;

               MacData var166;
               try {
                  var165.<init>(var106, password1);
                  var166 = new MacData;
               } catch (Exception var17) {
                  var161 = var17;
                  var133 = false;
                  break label576;
               }

               password2 = var166;

               try {
                  var166.<init>(var107, var80, var93);
                  break label308;
               } catch (Exception var16) {
                  var161 = var16;
                  var133 = false;
               }
            }

            Exception var81 = var161;
            throw new IOException("error constructing MAC: " + var81.toString());
         }

         Pfx var82;
         var82 = new Pfx.<init>(var89, password2);
         (new BEROutputStream(stream)).writeObject(var82);
      }
   }

   public static class BCPKCS12KeyStore extends JDKPKCS12KeyStore {
      public BCPKCS12KeyStore() {
         super("INFOSEC");
      }
   }

   private class CertId {
      byte[] id;

      CertId(PublicKey key) {
         this.id = JDKPKCS12KeyStore.this.createSubjectKeyId(key).getKeyIdentifier();
      }

      CertId(byte[] id) {
         this.id = id;
      }

      public int hashCode() {
         return Arrays.hashCode(this.id);
      }

      public boolean equals(Object o) {
         if (o == this) {
            return true;
         } else if (!(o instanceof CertId)) {
            return false;
         } else {
            CertId var10000 = this;
            CertId var2 = (CertId)o;
            return Arrays.areEqual(var10000.id, var2.id);
         }
      }
   }

   public static class DefPKCS12KeyStore extends JDKPKCS12KeyStore {
      public DefPKCS12KeyStore() {
         super((String)null);
      }
   }

   private static class IgnoresCaseHashtable {
      private Hashtable orig;
      private Hashtable keys;

      private IgnoresCaseHashtable() {
         IgnoresCaseHashtable var10000 = this;
         IgnoresCaseHashtable var10001 = this;
         super();
         Hashtable var1;
         var1 = new Hashtable.<init>();
         var10001.orig = var1;
         var1 = new Hashtable.<init>();
         var10000.keys = var1;
      }

      // $FF: synthetic method
      IgnoresCaseHashtable(IgnoresCaseHashtable var1) {
         this();
      }

      public void put(String key, Object value) {
         String var3 = Strings.toLowerCase(key);
         String var4;
         if ((var4 = (String)this.keys.get(var3)) != null) {
            this.orig.remove(var4);
         }

         this.keys.put(var3, key);
         this.orig.put(key, value);
      }

      public Enumeration keys() {
         return this.orig.keys();
      }

      public Object remove(String alias) {
         return (alias = (String)this.keys.remove(Strings.toLowerCase(alias))) == null ? null : this.orig.remove(alias);
      }

      public Object get(String alias) {
         return (alias = (String)this.keys.get(Strings.toLowerCase(alias))) == null ? null : this.orig.get(alias);
      }

      public Enumeration elements() {
         return this.orig.elements();
      }
   }
}
